<?php

namespace Drupal\wcm_simplesamlphp_auth\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\user\Entity\Role;

/**
 * Form builder for the simplesamlphp_auth basic settings form.
 */
class SettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'wcm_simplesamlphp_auth_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['wcm_simplesamlphp_auth.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $excludedRoles = ["anonymous","authenticated","administrator"];
    $config = $this->config('wcm_simplesamlphp_auth.settings');

    $allRoles = Role::loadMultiple();
    $roleNames = array_keys($allRoles);
    $roleValues = array('none' => 'none');
    foreach ($allRoles as $roleName => $role)
    {
        if (!in_array($roleName, $excludedRoles)) {
            $roleValues[$roleName] = $role->label();
        }
    }

    $form['basic'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('WCM SimplesamlPHP Auth Settings'),
      '#collapsible' => FALSE,
    ];
    $form['basic']['default_role'] = [
      '#type' => 'select',
      '#title' => $this->t('Default Role'),
      '#default_value' => $config->get('default_role'),
      '#options' => $roleValues,
      '#description' => "Additional role to assign to all authenticated users.",
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);
    $config = $this->config('wcm_simplesamlphp_auth.settings');

    $config->set('default_role', $form_state->getValue('default_role'));
    $config->save();
  }

}
